"""
Auto-generated class for NetworkMemberConfig
"""

from . import client_support


class NetworkMemberConfig(object):
    """
    auto-generated. don't touch.
    """

    @staticmethod
    def create(activeBridge=None, address=None, authorized=None, clock=None, identity=None, ipAssignments=None, memberRevision=None, nwid=None):
        """
        :type activeBridge: bool
        :type address: str
        :type authorized: bool
        :type clock: int
        :type identity: str
        :type ipAssignments: list[str]
        :type memberRevision: int
        :type nwid: str
        :rtype: NetworkMemberConfig
        """

        return NetworkMemberConfig(
            activeBridge=activeBridge,
            address=address,
            authorized=authorized,
            clock=clock,
            identity=identity,
            ipAssignments=ipAssignments,
            memberRevision=memberRevision,
            nwid=nwid,
        )

    def __init__(self, json=None, **kwargs):
        if not json and not kwargs:
            raise ValueError('No data or kwargs present')

        class_name = 'NetworkMemberConfig'
        create_error = '{cls}: unable to create {prop} from value: {val}: {err}'
        required_error = '{cls}: missing required property {prop}'

        data = json or kwargs

        property_name = 'activeBridge'
        val = data.get(property_name)
        if val is not None:
            datatypes = [bool]
            try:
                self.activeBridge = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'address'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.address = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'authorized'
        val = data.get(property_name)
        if val is not None:
            datatypes = [bool]
            try:
                self.authorized = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'clock'
        val = data.get(property_name)
        if val is not None:
            datatypes = [int]
            try:
                self.clock = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'identity'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.identity = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'ipAssignments'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.ipAssignments = client_support.list_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'memberRevision'
        val = data.get(property_name)
        if val is not None:
            datatypes = [int]
            try:
                self.memberRevision = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'nwid'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.nwid = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

    def __str__(self):
        return self.as_json(indent=4)

    def as_json(self, indent=0):
        return client_support.to_json(self, indent=indent)

    def as_dict(self):
        return client_support.to_dict(self)



class UserService:
    def __init__(self, client):
        self.client = client



    def listUsers(self, headers=None, query_params=None):
        """
        List viewable or editable users (including yourself).
        It is method for GET /user
        """
        uri = self.client.base_url + "/user"
        return self.client.get(uri, headers=headers, params=query_params)


    def getUser(self, id, headers=None, query_params=None):
        """
        Get a user by UUID. Get a user by UUID
        It is method for GET /user/{id}
        """
        uri = self.client.base_url + "/user/"+id
        return self.client.get(uri, headers=headers, params=query_params)


    def updateUser(self, data, id, headers=None, query_params=None):
        """
        Update user information.
        It is method for POST /user/{id}
        """
        uri = self.client.base_url + "/user/"+id
        return self.client.post(uri, data, headers=headers, params=query_params)



import requests

from .network_service import  NetworkService 
from .self_service import  SelfService 
from .status_service import  StatusService 
from .user_service import  UserService 


class Client:
    def __init__(self, base_uri = "https://my.zerotier.com/api"):
        self.base_url = base_uri
        self.session = requests.Session()
        self.session.headers.update({"Content-Type": "application/json"})
        
        self.network = NetworkService(self)
        self.self = SelfService(self)
        self.status = StatusService(self)
        self.user = UserService(self)

    def is_goraml_class(self, data):
        # check if a data is go-raml generated class
        # we currently only check the existence
        # of as_json method
        op = getattr(data, "as_json", None)
        if callable(op):
            return True
        return False

    def set_auth_header(self, val):
        ''' set authorization header value'''
        self.session.headers.update({"Authorization":val})

    def get(self, uri, headers, params):
        res = self.session.get(uri, headers=headers, params=params)
        res.raise_for_status()
        return res

    def post(self, uri, data, headers, params):
        if self.is_goraml_class(data):
            res = self.session.post(uri, data=data.as_json(), headers=headers, params=params)
        elif type(data) is str:
            res = self.session.post(uri, data=data, headers=headers, params=params)
        else:
            res = self.session.post(uri, json=data, headers=headers, params=params)
        res.raise_for_status()
        return res

    def put(self, uri, data, headers, params):
        if self.is_goraml_class(data):
            res = self.session.put(uri, data=data.as_json(), headers=headers, params=params)
        elif type(data) is str:
            res = self.session.put(uri, data=data, headers=headers, params=params)
        else:
            res = self.session.put(uri, json=data, headers=headers, params=params)
        res.raise_for_status()
        return res

    def patch(self, uri, data, headers, params):
        if self.is_goraml_class(data):
            res = self.session.patch(uri, data=data.as_json(), headers=headers, params=params)
        elif type(data) is str:
            res = self.session.patch(uri, data=data, headers=headers, params=params)
        else:
            res = self.session.patch(uri, json=data, headers=headers, params=params)
        res.raise_for_status()
        return res


import datetime
import time


def generate_rfc3339(d, local_tz=True):
    """
    generate rfc3339 time format
    input :
    d = date type
    local_tz = use local time zone if true,
    otherwise mark as utc

    output :
    rfc3339 string date format. ex : `2008-04-02T20:00:00+07:00`
    """
    try:
        if local_tz:
            d = datetime.datetime.fromtimestamp(d)
        else:
            d = datetime.datetime.utcfromtimestamp(d)
    except TypeError:
        pass

    if not isinstance(d, datetime.date):
        raise TypeError('Not timestamp or date object. Got %r.' % type(d))

    if not isinstance(d, datetime.datetime):
        d = datetime.datetime(*d.timetuple()[:3])

    return ('%04d-%02d-%02dT%02d:%02d:%02d%s' %
            (d.year, d.month, d.day, d.hour, d.minute, d.second,
             _generate_timezone(d, local_tz)))


def _calculate_offset(date, local_tz):
    """
    input :
    date : date type
    local_tz : if true, use system timezone, otherwise return 0

    return the date of UTC offset.
    If date does not have any timezone info, we use local timezone,
    otherwise return 0
    """
    if local_tz:
        #handle year before 1970 most sytem there is no timezone information before 1970.
        if date.year < 1970:
            # Use 1972 because 1970 doesn't have a leap day
            t = time.mktime(date.replace(year=1972).timetuple)
        else:
            t = time.mktime(date.timetuple())

        # handle daylightsaving, if daylightsaving use altzone, otherwise use timezone
        if time.localtime(t).tm_isdst:
            return -time.altzone
        else:
            return -time.timezone
    else:
        return 0


def _generate_timezone(date, local_tz):
    """
    input :
    date : date type
    local_tz : bool

    offset generated from _calculate_offset
    offset in seconds
    offset = 0 -> +00:00
    offset = 1800 -> +00:30
    offset = -3600 -> -01:00
    """
    offset = _calculate_offset(date, local_tz)

    hour = abs(offset) // 3600
    minute = abs(offset) % 3600 // 60

    if offset < 0:
        return '%c%02d:%02d' % ("-", hour, minute)
    else:
        return '%c%02d:%02d' % ("+", hour, minute)


class SelfService:
    def __init__(self, client):
        self.client = client



    def getAuthenticatedUser(self, headers=None, query_params=None):
        """
        Get currently authenticated user (if any).
        It is method for GET /self
        """
        uri = self.client.base_url + "/self"
        return self.client.get(uri, headers=headers, params=query_params)



class NetworkService:
    def __init__(self, client):
        self.client = client



    def listNetworks(self, headers=None, query_params=None):
        """
        Get a list of networks this user owns or can view/edit
        It is method for GET /network
        """
        uri = self.client.base_url + "/network"
        return self.client.get(uri, headers=headers, params=query_params)


    def createNetwork(self, data, headers=None, query_params=None):
        """
        Create a new network
        It is method for POST /network
        """
        uri = self.client.base_url + "/network"
        return self.client.post(uri, data, headers=headers, params=query_params)


    def getNetwork(self, id, headers=None, query_params=None):
        """
        Get network configuration and status information
        It is method for GET /network/{id}
        """
        uri = self.client.base_url + "/network/"+id
        return self.client.get(uri, headers=headers, params=query_params)


    def updateNetwork(self, data, id, headers=None, query_params=None):
        """
        Update network configuration
        It is method for POST /network/{id}
        """
        uri = self.client.base_url + "/network/"+id
        return self.client.post(uri, data, headers=headers, params=query_params)


    def deleteNetwork(self, id, headers=None, query_params=None):
        """
        Delete network
        It is method for DELETE /network/{id}
        """
        uri = self.client.base_url + "/network/"+id
        return self.client.session.delete(uri, headers=headers, params=query_params)


    def listMembers(self, id, headers=None, query_params=None):
        """
        Get a list of network members
        It is method for GET /network/{id}/member
        """
        uri = self.client.base_url + "/network/"+id+"/member"
        return self.client.get(uri, headers=headers, params=query_params)


    def getMember(self, address, id, headers=None, query_params=None):
        """
        Get network member settings
        It is method for GET /network/{id}/member/{address}
        """
        uri = self.client.base_url + "/network/"+id+"/member/"+address
        return self.client.get(uri, headers=headers, params=query_params)


    def updateMember(self, data, address, id, headers=None, query_params=None):
        """
        Create or update member settings
        It is method for POST /network/{id}/member/{address}
        """
        uri = self.client.base_url + "/network/"+id+"/member/"+address
        return self.client.post(uri, data, headers=headers, params=query_params)



"""
Auto-generated class for SystemStatus
"""
from .User import User

from . import client_support


class SystemStatus(object):
    """
    auto-generated. don't touch.
    """

    @staticmethod
    def create(clock=None, loginMethods=None, online=None, paidPlans=None, smsEnabled=None, stripePublishableKey=None, uptime=None, user=None, version=None):
        """
        :type clock: int
        :type loginMethods: dict
        :type online: bool
        :type paidPlans: dict
        :type smsEnabled: bool
        :type stripePublishableKey: str
        :type uptime: int
        :type user: User
        :type version: str
        :rtype: SystemStatus
        """

        return SystemStatus(
            clock=clock,
            loginMethods=loginMethods,
            online=online,
            paidPlans=paidPlans,
            smsEnabled=smsEnabled,
            stripePublishableKey=stripePublishableKey,
            uptime=uptime,
            user=user,
            version=version,
        )

    def __init__(self, json=None, **kwargs):
        if not json and not kwargs:
            raise ValueError('No data or kwargs present')

        class_name = 'SystemStatus'
        create_error = '{cls}: unable to create {prop} from value: {val}: {err}'
        required_error = '{cls}: missing required property {prop}'

        data = json or kwargs

        property_name = 'clock'
        val = data.get(property_name)
        if val is not None:
            datatypes = [int]
            try:
                self.clock = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'loginMethods'
        val = data.get(property_name)
        if val is not None:
            datatypes = [dict]
            try:
                self.loginMethods = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'online'
        val = data.get(property_name)
        if val is not None:
            datatypes = [bool]
            try:
                self.online = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'paidPlans'
        val = data.get(property_name)
        if val is not None:
            datatypes = [dict]
            try:
                self.paidPlans = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'smsEnabled'
        val = data.get(property_name)
        if val is not None:
            datatypes = [bool]
            try:
                self.smsEnabled = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'stripePublishableKey'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.stripePublishableKey = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'uptime'
        val = data.get(property_name)
        if val is not None:
            datatypes = [int]
            try:
                self.uptime = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'user'
        val = data.get(property_name)
        if val is not None:
            datatypes = [User]
            try:
                self.user = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'version'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.version = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

    def __str__(self):
        return self.as_json(indent=4)

    def as_json(self, indent=0):
        return client_support.to_json(self, indent=indent)

    def as_dict(self):
        return client_support.to_dict(self)



"""
Auto-generated class for NetworkConfig
"""
from .NetworkIpAssignmentPool import NetworkIpAssignmentPool
from .NetworkRelay import NetworkRelay
from .NetworkRule import NetworkRule

from . import client_support


class NetworkConfig(object):
    """
    auto-generated. don't touch.
    """

    @staticmethod
    def create(authorizedMemberCount=None, clock=None, controllerInstanceId=None, creationTime=None, ipAssignmentPools=None, ipLocalRoutes=None, memberRevisionCounter=None, multicastLimit=None, name=None, nwid=None, private=None, relays=None, revision=None, rules=None, v4AssignMode=None, v6AssignMode=None):
        """
        :type authorizedMemberCount: int
        :type clock: int
        :type controllerInstanceId: str
        :type creationTime: int
        :type ipAssignmentPools: list[NetworkIpAssignmentPool]
        :type ipLocalRoutes: list[str]
        :type memberRevisionCounter: int
        :type multicastLimit: int
        :type name: str
        :type nwid: str
        :type private: bool
        :type relays: list[NetworkRelay]
        :type revision: int
        :type rules: list[NetworkRule]
        :type v4AssignMode: str
        :type v6AssignMode: str
        :rtype: NetworkConfig
        """

        return NetworkConfig(
            authorizedMemberCount=authorizedMemberCount,
            clock=clock,
            controllerInstanceId=controllerInstanceId,
            creationTime=creationTime,
            ipAssignmentPools=ipAssignmentPools,
            ipLocalRoutes=ipLocalRoutes,
            memberRevisionCounter=memberRevisionCounter,
            multicastLimit=multicastLimit,
            name=name,
            nwid=nwid,
            private=private,
            relays=relays,
            revision=revision,
            rules=rules,
            v4AssignMode=v4AssignMode,
            v6AssignMode=v6AssignMode,
        )

    def __init__(self, json=None, **kwargs):
        if not json and not kwargs:
            raise ValueError('No data or kwargs present')

        class_name = 'NetworkConfig'
        create_error = '{cls}: unable to create {prop} from value: {val}: {err}'
        required_error = '{cls}: missing required property {prop}'

        data = json or kwargs

        property_name = 'authorizedMemberCount'
        val = data.get(property_name)
        if val is not None:
            datatypes = [int]
            try:
                self.authorizedMemberCount = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'clock'
        val = data.get(property_name)
        if val is not None:
            datatypes = [int]
            try:
                self.clock = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'controllerInstanceId'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.controllerInstanceId = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'creationTime'
        val = data.get(property_name)
        if val is not None:
            datatypes = [int]
            try:
                self.creationTime = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'ipAssignmentPools'
        val = data.get(property_name)
        if val is not None:
            datatypes = [NetworkIpAssignmentPool]
            try:
                self.ipAssignmentPools = client_support.list_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'ipLocalRoutes'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.ipLocalRoutes = client_support.list_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'memberRevisionCounter'
        val = data.get(property_name)
        if val is not None:
            datatypes = [int]
            try:
                self.memberRevisionCounter = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'multicastLimit'
        val = data.get(property_name)
        if val is not None:
            datatypes = [int]
            try:
                self.multicastLimit = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'name'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.name = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'nwid'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.nwid = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'private'
        val = data.get(property_name)
        if val is not None:
            datatypes = [bool]
            try:
                self.private = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'relays'
        val = data.get(property_name)
        if val is not None:
            datatypes = [NetworkRelay]
            try:
                self.relays = client_support.list_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'revision'
        val = data.get(property_name)
        if val is not None:
            datatypes = [int]
            try:
                self.revision = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'rules'
        val = data.get(property_name)
        if val is not None:
            datatypes = [NetworkRule]
            try:
                self.rules = client_support.list_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'v4AssignMode'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.v4AssignMode = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'v6AssignMode'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.v6AssignMode = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

    def __str__(self):
        return self.as_json(indent=4)

    def as_json(self, indent=0):
        return client_support.to_json(self, indent=indent)

    def as_dict(self):
        return client_support.to_dict(self)



from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

setup (
    name='zerotier',
    version='1.0.0',
    description='Zerotier API client',
    url='https://github.com/g8os/zerotier_client',
    author='Christophe de Carvalho',
    author_email='christophe@gig.tech',
    license='Apache 2.0',
    packages=find_packages(),
    install_requires=['requests>=2.13.0'],
)



from enum import Enum


class EnumUserAuth(Enum):
    local = "local"
    google = "google"
    github = "github"
    slack = "slack"
    facebook = "facebook"
    twitter = "twitter"
    linkedin = "linkedin"
